# '==================================================================================================================================================================
# 'Disclaimer
# 'The sample scripts are not supported under any N-Able support program or service.
# 'The sample scripts are provided AS IS without warranty of any kind.
# 'N-Able further disclaims all implied warranties including, without limitation, any implied warranties of merchantability or of fitness for a particular purpose.
# 'The entire risk arising out of the use or performance of the sample scripts and documentation stays with you.
# 'In no event shall N-Able or anyone else involved in the creation, production, or delivery of the scripts be liable for any damages whatsoever
# '(including, without limitation, damages for loss of business profits, business interruption, loss of business information, or other pecuniary loss)
# 'arising out of the use of or inability to use the sample scripts or documentation.
# '==================================================================================================================================================================

Param (
    [string]$verbose = "Y"
)

function setupLogging() {
    $script:logFilePath = "C:\ProgramData\MspPlatform\Tech Tribes\Ecosystem Cleanup\debug.log"

    $script:logFolder = Split-Path $logFilePath
    $script:logFile = Split-Path $logFilePath -Leaf

    If (($logFolder -match '.+?\\$') -eq $false) {
        $script:logFolder = $script:logFolder + "\"
    }

    $logFolderExists = Test-Path $logFolder
    $logFileExists = Test-Path $logFilePath

    If ($logFolderExists -eq $false) {
        New-Item -ItemType "directory" -Path $logFolder | Out-Null
    }

    If ($logFileExists -eq $true) {
        Remove-Item $logFilePath -ErrorAction SilentlyContinue
        Start-Sleep 2
        New-Item -ItemType "file" -Path $logFolder -Name $logFile | Out-Null
    } Else {
        New-Item -ItemType "file" -Path $logFolder -Name $logFile | Out-Null
    }
}

function validateUserInput() {
    # Ensures the provided input from user is valid
    If ($verbose.ToLower() -eq "y") {
        $script:verboseMode = $true
        writeToLog V "You have defined to have the script output the verbose log entries."
    } Else {
        $script:verboseMode = $false
        writeToLog I "Will output logs in regular mode."
    }

    writeToLog V "Input Parameters have been successfully validated."
    writeToLog V ("### Completed running {0} function. ###`r`n" -f $MyInvocation.MyCommand)
}
    
function stopServices() {
    writeToLog V ("### Started running {0} function. ###`r`n" -f $MyInvocation.MyCommand)

    try {
        Get-Service -DisplayName "Windows Agent Service" | Stop-Service -WarningAction SilentlyContinue -ErrorAction Stop
    }
    catch {
        $msg = $_.Exception.Message
        $line = $_.InvocationInfo.ScriptLineNumber
        writeToLog F "Error occurred whilst attempting to stop the Windows Agent service, due to:`r`n`t$msg"
        writeToLog V "This occurred on line number: $line"
        writeToLog F "Failing script."
        Exit 1001
    }

    try {
        Get-Service -DisplayName "Windows Agent Maintenance Service" | Stop-Service -WarningAction SilentlyContinue -ErrorAction Stop
    }
    catch {
        $msg = $_.Exception.Message
        $line = $_.InvocationInfo.ScriptLineNumber
        writeToLog F "Error occurred whilst attempting to stop the Windows Agent Maintenance service, due to:`r`n`t$msg"
        writeToLog V "This occurred on line number: $line"
        writeToLog F "Failing script."
        Exit 1001
    }

    writeToLog V ("### Completed running {0} function. ###`r`n" -f $MyInvocation.MyCommand)
}

function uninstallAgent() {
    writeToLog V ("### Started running {0} function. ###`r`n" -f $MyInvocation.MyCommand)

    $ecosystemUninstaller = "C:\Program Files (x86)\SolarWinds MSP\Ecosystem Agent\unins000.exe"

    try {
        Start-Process -Wait -FilePath $ecosystemUninstaller -ArgumentList "/VERYSILENT" -PassThru
    }
    catch {
        $msg = $_.Exception.Message
		$line = $_.InvocationInfo.ScriptLineNumber

        writeToLog F "Failed to uninstall Ecosystem Agent, failure occurred due to:`r`n`t$msg"
        writeToLog V "This occurred on line number: $line"
        writeToLog F "Failing script."
        Exit 1001
    }

    $array = @()
	$array += "C:\Program Files (x86)\N-Able Technologies\Windows Agent\config\ExecutionerConfig.xml"
    $array += "C:\Program Files (x86)\N-Able Technologies\Windows Agent\config\ExecutionerConfig.xml.backup"
    $array += "C:\Program Files (x86)\N-Able Technologies\Windows Agent\config\ExecutionerErrorManager.xml"
    $array += "C:\Program Files (x86)\N-Able Technologies\Windows Agent\config\ExecutionerErrorManager.xml.backup"

    foreach ($xml in $array) {
        try {
            Remove-Item -Path $xml -Recurse
        }
        catch {
            $msg = $_.Exception.Message
            $line = $_.InvocationInfo.ScriptLineNumber

            writeToLog F "Failed to delete $xml, due to:`r`n`t$msg"
            writeToLog V "This occurred on line number: $line"
            writeToLog F "Failing script."
            Exit 1001
        }
    }

    writeToLog V ("### Completed running {0} function. ###`r`n" -f $MyInvocation.MyCommand)
}

function startServices() {
    writeToLog V ("### Started running {0} function. ###`r`n" -f $MyInvocation.MyCommand)

    try {
        Get-Service -DisplayName "Windows Agent Service" | Start-Service -WarningAction SilentlyContinue -ErrorAction Stop
    }
    catch {
        $msg = $_.Exception.Message
        $line = $_.InvocationInfo.ScriptLineNumber
        writeToLog F "Error occurred whilst attempting to start the Windows Agent service, due to:`r`n`t$msg"
        writeToLog V "This occurred on line number: $line"
        writeToLog F "Failing script."
        Exit 1001
    }

    try {
        Get-Service -DisplayName "Windows Agent Maintenance Service" | Start-Service -WarningAction SilentlyContinue -ErrorAction Stop
    }
    catch {
        $msg = $_.Exception.Message
        $line = $_.InvocationInfo.ScriptLineNumber
        writeToLog F "Error occurred whilst attempting to start the Windows Agent Maintenance service, due to:`r`n`t$msg"
        writeToLog V "This occurred on line number: $line"
        writeToLog F "Failing script."
        Exit 1001
    }

    writeToLog V ("### Completed running {0} function. ###`r`n" -f $MyInvocation.MyCommand)
}

function writeToLog($state, $message) {

    $script:timestamp = "[{0:dd/MM/yy} {0:HH:mm:ss}]" -f (Get-Date)

    switch -regex -Wildcard ($state) {
        "I" {
            $state = "INFO"
        }
        "E" {
            $state = "ERROR"
        }
        "W" {
            $state = "WARNING"
        }
        "F"  {
            $state = "FAILURE"
        }
        "V"  {
            If ($verboseMode -eq $true) {
                $state = "VERBOSE"
            } Else {
                return
            }
        }
        ""  {
            $state = "INFO"
        }
        Default {
            $state = "INFO"
        }
     }

    Write-Host "$($timeStamp) - [$state]: $message"
    Write-Output "$($timeStamp) - [$state]: $message" | Out-file $logFilePath -Append
}

function main() {
    setupLogging
    validateUserInput
    stopServices
    uninstallAgent
    startServices
    
    writeToLog I "Ecosystem Agent is now uninstalled. The agent will automatically reinstall shortly."
}

main